@@ -40,10 +40,9 @@
 #define DBL_MAX 1.79769313486e+308
 #endif
 #define PLASTIC_NUMBER 1.32471795724
-#define PLASTIC_NUMBER_SQUARED 1.75487766625
-#define PLASTIC_NUMBER_CUBED 2.32471795724
+#define PLASTIC_NUMBER_2 1.75487766625
+#define PLASTIC_NUMBER_3 2.32471795724
 #define MARGIN_FACTOR 0.139680581996 // pow(PLASTIC_NUMBER,-7)
-#define NEGLIGIBILITY_FACTOR 0.0195106649868 // MARGIN_FACTOR^2
 
 #ifndef NO_NAMESPACE_GIAC
 namespace giac {
@@ -101,10 +100,10 @@ public:
     typedef std::vector<std::map<int,double> > edgemapd;
     typedef std::map<ipair,int> intpoly;
     typedef std::vector<double> dvector;
+    typedef std::set<int> iset;
 
     class vertex { // vertex class
         int m_subgraph;
-        bool m_sorted;
         // used for traversing
         bool m_visited;
         int m_low;
@@ -131,7 +130,6 @@ public:
         gen label() const;
         inline bool supports_attributes() const { return m_attributes!=NULL; }
         inline void unsupport_attributes() { m_attributes=NULL; m_neighbor_attributes=NULL; }
-        inline bool is_sorted() const { return m_sorted; }
         inline void set_label(const gen &s) { assert(supports_attributes()); (*m_attributes)[_GT_ATTRIB_LABEL]=s; }
         inline void set_subgraph(int s) { m_subgraph=s; }
         inline int subgraph() const { return m_subgraph; }
@@ -155,15 +153,13 @@ public:
         inline void set_attribute(int key,const gen &val) { assert(supports_attributes()); (*m_attributes)[key]=val; }
         inline void set_attributes(const attrib &attr) { assert(supports_attributes()); copy_attributes(attr,*m_attributes); }
         inline const ivector &neighbors() const { return m_neighbors; }
+        inline int degree() const { return m_neighbors.size(); }
         void add_neighbor(int i,const attrib &attr=attrib());
         bool is_temporary(int i) const;
         attrib &neighbor_attributes(int i);
         const attrib &neighbor_attributes(int i) const;
-        bool has_neighbor(int i) const;
-        void move_neighbor(int i,int j,bool after=true);
+        inline bool has_neighbor(int i) const { return binary_search(m_neighbors.begin(),m_neighbors.end(),i); }
         void remove_neighbor(int i);
-        template<class Compare>
-        inline void sort_neighbors(Compare comp) { sort(m_neighbors.begin(),m_neighbors.end(),comp); m_sorted=false; }
         inline void clear_neighbors();
         void incident_faces(ivector &F) const;
         inline void add_edge_face(int nb,int f) { assert(m_faces.find(nb)==m_faces.end()); m_faces[nb]=f+1; }
@@ -179,9 +175,7 @@ public:
 
     class dotgraph { // temporary structure used in dot parsing
         int m_index;
-        attrib vertex_attr;
-        attrib edge_attr;
-        attrib chain_attr;
+        attrib vertex_attr,edge_attr,chain_attr;
         ivector m_chain;
         int pos;
     public:
@@ -210,16 +204,9 @@ public:
     class walker { // tree node positioner
         graphe *G;
         layout *x;
-        double hsep;
-        double vsep;
+        double hsep,vsep;
         ivectors levels;
-        ivector node_counters;
-        ivector gap_counters;
-        ivector position;
-        dvector prelim;
-        dvector modifier;
-        ivector gaps;
-        ivector children;
+        ivector node_counters,gap_counters,position,prelim,modifier,gaps,children;
         std::queue<int> placed;
         int depth;
         void walk(int i,int pass,int level=0,double modsum=0);
@@ -229,31 +216,32 @@ public:
         void positioning(int apex);
     };
 
+    class circ_enum { // circuit enumeration in digraphs
+        graphe *G;
+        ivector point_stack;
+        std::stack<int> marked_stack;
+        ivectors A,res;
+        std::vector<bool> mark;
+        int s,lb,ub;
+        void backtrack(int v,bool &f);
+    public:
+        circ_enum(graphe *gr,int lo=-1,int hi=-1);
+        ivectors find_cycles();
+    };
+
 #ifdef HAVE_LIBGLPK
     class painter { // vertex painter
         graphe *G;
         ivectors values;
-        ivector cover_number;
-        ivector initially_colored;
         ipairs col2ij;
-        std::vector<bool> iscliq;
-        ivector branch_candidates;
-        std::vector<double> fracts;
-        ivector temp_colors;
-        ivector ordering;
+        std::vector<bool> iscliq,fracts;
+        ivector cover_number,initially_colored,branch_candidates,temp_colors,ordering;
         std::set<int> used_colors;
-        int lb;
-        int ub;
-        int maxiter;
-        int nxcols;
+        int lb,ub,maxiter,nxcols;
         bool generate_clique_cuts;
         glp_prob *mip;
-        double timeout;
-        double *heur;
-        int *row_indices;
-        double *row_coeffs;
-        int *best_indices;
-        double *best_coeffs;
+        double timeout,*heur,*row_coeffs,*best_coeffs;
+        int *row_indices,*best_indices;
         void compute_bounds(const ivector &icol,int max_colors);
         void make_values();
         void formulate_mip();
@@ -293,8 +281,8 @@ public:
         bool isweighted;                        // true iff G is weighted
         int sg;                                 // current subgraph index
         std::set<ivector> subtours;             // subtours collected in during solving the last MIP
-        ivectors hierarhical_clustering_forest; // hierarhical clustering forest of subgraphs
-        ivector tour;                           // a tour
+        ivectors clustering_forest;             // hierarchical clustering forest of subgraphs
+        ivector tour,old_sol;                   // a tour, old mip solution
         double *coeff;                          // coefficients to be passed to MIP solver
         int *indices;                           // indices of row entries to be passed to MIP solver
         bool *visited;                          // used to mark vertices as visited
@@ -317,15 +305,15 @@ public:
         dvector obj;
         std::vector<bool> can_branch;
         void formulate_mip();
-        void get_subtours();
+        bool get_subtours();
         void add_subtours(const ivectors &sv);
         void lift_subtours(ivectors &sv) const;
         bool find_subgraph_subtours(ivectors &sv,solution_status &status);
         bool subtours_equal(const ivector &st1,const ivector &st2);
         ivector canonical_subtour(const ivector &subtour);
         void append_sce(const ivector &subtour);
-        void make_hierarhical_clustering_forest();
-        void hierarhical_clustering_dfs(int i,ivectors &considered_sec,ivectors &relevant_sec);
+        void make_hierarchical_clustering_forest();
+        void hierarchical_clustering_dfs(int i,ivectors &considered_sec,ivectors &relevant_sec);
         ipair make_edge(int i,int j) const;
         void make_sg_edges();
         int edge_index(const ipair &e);
@@ -369,12 +357,8 @@ public:
 #endif
 
     class rectangle { // simple rectangle class
-        double m_x;
-        double m_y;
-        double m_width;
-        double m_height;
-        bool m_locked_above;
-        bool m_locked_right;
+        double m_x,m_y,m_width,m_height;
+        bool m_locked_above,m_locked_right;
         layout *L;
     public:
         struct comparator {
@@ -435,16 +419,43 @@ public:
 #endif
     };
 
+    class ransampl { // random sampling from a given degree distribution
+        int n;
+        vecteur prob;
+        ivector alias;
+        const context *ctx;
+    public:
+        ransampl(const vecteur &p,GIAC_CONTEXT);
+        gen data() const;
+        int generate() const;
+    };
+
+    class bucketsampler { // random sampling from a dynamic distribution
+        const context *ctx;
+        ivector weights;
+        long total_weight;
+        std::map<int,int> max_weight,level_weight;
+        std::map<int,ivector> levels;
+        ipairs positions;
+        inline int nearest_pow2(double a) { return std::floor(0.5+std::log(a)/M_LN2); }
+    public:
+        bucketsampler(const ivector &W,GIAC_CONTEXT);
+        int generate();
+        void insert(int w);
+        void update(int i,int w);
+        inline void increment(int i) { update(i,weights[i]+1); }
+    };
+
     class unionfind { // disjoint-set data structure
         struct element {
-            int id;
-            int parent;
-            int rank;
+            int id,parent,rank;
             element() { id=-1; }
         };
-        std::vector<element> elements;
+        int sz;
+        element *elements;
     public:
-        unionfind(int n) { elements.resize(n); }
+        unionfind(int n) { sz=n; elements=new element[n]; }
+        ~unionfind() { delete[] elements; }
         void make_set(int id);
         bool is_stored(int id);
         int find(int id);
@@ -455,10 +466,8 @@ public:
         graphe *G;
         int maxsize;
         bool found;
-        double timeout; // in seconds
-        ivector c;
-        ivector incumbent;
-        ivector clique_nodes;
+        double timeout; // seconds
+        ivector c,incumbent,clique_nodes;
         clock_t start;
         bool timed_out;
         void recurse(ivector &U,int size,ivector &position);
@@ -467,7 +476,7 @@ public:
         int maxclique(ivector &clique);
     };
 
-    struct edges_comparator { // sort edges by their weight
+    struct edges_comparator { // for sorting edges by their weight
         graphe *G;
         bool operator()(const ipair &a,const ipair &b) {
             return is_strictly_greater(G->weight(b),G->weight(a),G->giac_context());
@@ -475,13 +484,13 @@ public:
         edges_comparator(graphe *gr) { G=gr; }
     };
 
-    struct ivectors_comparator { // sort ivectors by their length
+    struct ivectors_comparator { // for sorting ivectors by their length
         bool operator()(const ivector &a,const ivector &b) {
             return a.size()<b.size();
         }
     };
 
-    struct degree_comparator { // sort vertices by their degrees
+    struct degree_comparator { // for sorting vertices by their degrees
         graphe *G;
         bool asc;
         bool operator()(int v,int w) {
@@ -491,7 +500,7 @@ public:
         degree_comparator(graphe *gr,bool ascending=true) { G=gr; asc=ascending; }
     };
 
-    struct ivectors_degree_comparator { // sort sets of vertices by ascending total degree
+    struct ivectors_degree_comparator { // for sorting sets of vertices by ascending total degree
         graphe *G;
         bool operator()(const ivector &a,const ivector &b) {
             int deg_a,deg_b;
@@ -506,6 +515,14 @@ public:
         ivectors_degree_comparator(graphe *gr) { G=gr; }
     };
 
+    struct kspaths_comparator { // for sorting paths by their weight
+        bool operator()(const std::pair<gen,ivector> &a,const std::pair<gen,ivector> &b) const {
+            if (is_zero(a.first-b.first))
+                return a.second<b.second;
+            return is_strictly_greater(b.first,a.first,context0);
+        }
+    };
+
     typedef std::vector<vertex>::const_iterator node_iter;
     typedef std::map<int,attrib>::const_iterator neighbor_iter;
     typedef attrib::const_iterator attrib_iter;
@@ -516,6 +533,8 @@ public:
     typedef edgeset::const_iterator edgeset_iter;
     typedef intpoly::const_iterator intpoly_iter;
     typedef dvector::const_iterator dvector_iter;
+    typedef iset::const_iterator iset_iter;
+
     static const gen FAUX;
     static const gen VRAI;
     static bool verbose;
@@ -536,6 +555,8 @@ public:
     static const int grotzsch_graph[];
     static const int harries_graph_lcf[];
     static const int harries_wong_graph_lcf[];
+    static const int balaban_10cage_lcf[];
+    static const int balaban_11cage_lcf[];
     static const int heawood_graph[];
     static const int herschel_graph[];
     static const int mcgee_graph[];
@@ -545,10 +566,31 @@ public:
     static const int tetrahedron_graph[];
     static const int octahedron_graph[];
     static const int icosahedron_graph[];
-    static const int levi_graph[];
     static const int ljubljana_graph_lcf[];
     static const int foster_graph_lcf[];
     static const int blanusa_graph[];
+    static const int bidiakis_cube_graph_lcf[];
+    static const int bull_graph[];
+    static const int butterfly_graph[];
+    static const int diamond_graph[];
+    static const int chvatal_graph[];
+    static const int franklin_graph_lcf[];
+    static const int frucht_graph_lcf[];
+    static const int biggs_smith_graph_lcf[];
+    static const int moser_spindle_graph[];
+    static const int errera_graph[];
+    static const int goldner_harary_graph[];
+    static const int golomb_graph[];
+    static const int hoffman_graph_matrix[8][8];
+    static const int poussin_graph[];
+    static const int wagner_graph[];
+    static const int folkman_graph_lcf[];
+    static const int gray_graph_lcf[];
+    static const int tutte_12cage_lcf[];
+    static const int tutte_8cage_lcf[];
+    static const int f26a_graph_lcf[];
+    static const int tietze_graph[];
+    static const int tutte_fragment_graph[];
 
 private:
     const context *ctx;
@@ -580,9 +622,12 @@ private:
     static void attrib2genmap(const attrib &attr,gen_map &m);
     static void copy_attributes(const attrib &src,attrib &dest);
     void write_attrib(std::ofstream &dotfile,const attrib &attr) const;
-    static int sets_union(const ivector &A,const ivector &B,ivector &U);
-    static int sets_intersection(const ivector &A,const ivector &B,ivector &I);
-    static int sets_difference(const ivector &A,const ivector &B,ivector &D);
+    static ivector_iter binsearch(ivector_iter first,ivector_iter last,int a);
+    static size_t sets_union(const iset &A,const iset &B,iset &U);
+    static size_t sets_intersection(const iset &A,const iset &B,iset &I);
+    static size_t sets_intersection(const ivector &A,const iset &B,iset &I);
+    static size_t sets_difference(const iset &A,const iset &B,iset &D);
+    static size_t sets_difference(const iset &A,const ivector &B,iset &D);
     static point make_point(double x,double y) { point p(2,x); p.back()=y; return p; }
     static point make_point(double x,double y,double z) { point p(3,x); p[1]=y; p.back()=z; return p; }
     static void add_point(point &a,const point &b);
@@ -610,13 +655,11 @@ private:
     void multilevel_recursion(layout &x,int d,double R,double K,double tol,int depth=0);
     int mdeg(const ivector &V,int i) const;
     void coarsening(graphe &G,const sparsemat &P,const ivector &V) const;
-    void tomita(ivector &R,ivector &P,ivector &X,std::map<int,int> &m,int mode=0);
+    void tomita(iset &R,iset &P,iset &X,std::map<int,int> &m,int mode=0);
     int cp_maxclique(ivector &clique);
     void cp_recurse(ivector &C,ivector &P,ivector &incumbent);
     int ost_maxclique(ivector &clique);
     void ost_recursive(ivector &U,int size,int &maxsize,ivector &incumbent,bool &found);
-    void remove_isolated_node(int i);
-    void remove_isolated_nodes(const std::set<int> &isolated_nodes);
     void find_cut_vertices_dfs(int i,std::set<int> &ap,int sg);
     void find_blocks_dfs(int i,std::vector<ipairs> &blocks,int sg);
     void find_bridges_dfs(int i,ipairs &B,int sg);
@@ -649,7 +692,6 @@ private:
     static int common_element(const ivector &v1,const ivector &v2,int offset=0);
     void embed_children_blocks(int i,ivectors &block_tree,std::vector<ivectors> &blocks_faces);
     void periphericity(const ivector &outer_face,ivector &p);
-    static void tree_layout2polar(layout &ly);
     void tree_height_dfs(int i,int level,int &depth);
     void make_product_nodes(const graphe &G,graphe &P) const;
     static void extract_path_from_cycle(const ivector &cycle,int i,int j,ivector &path);
@@ -669,7 +711,7 @@ private:
     int saturation_degree(const vertex &v,std::set<int> &colors) const;
     int uncolored_degree(const vertex &v) const;
     bool is_partially_colored() const;
-    void remove_maximal_clique(ivector &V) const;
+    void remove_maximal_clique(iset &V) const;
     bool bipartite_matching_bfs(ivector &dist);
     bool bipartite_matching_dfs(int u,ivector &dist);
     static ipair forest_root_info(const ivector &forest,int v);
@@ -691,6 +733,9 @@ private:
     void save_subgraphs();
     void restore_subgraphs();
     int vertex_pair_connectivity(int v,int w);
+    static inline gen harmonic_mean_exact(gen a,gen b,gen c) { return 3*a*b*c/(a*b+b*c+a*c); }
+    static inline double harmonic_mean(double a,double b,double c) { return 3.0*a*b*c/(a*b+b*c+a*c); }
+    void strec(int i,int t,int counter,int np,iset &Q,vecteur &timestamp,vecteur &l);
 
 public:
     graphe(const context *contextptr=context0,bool support_attributes=true);
@@ -705,6 +750,7 @@ public:
     inline const context *giac_context() const { return ctx; }
     static gen make_idnt(const char* name,int index=-1,bool intern=true);
     void make_default_labels(vecteur &labels,int n,int n0=0,int offset=-1) const;
+    bool labels2iset(const vecteur &labels,iset &s);
     static gen boole(bool b) { return b?VRAI:FAUX; }
     static gen word2gen(const std::string &word);
     static gen str2gen(const std::string &str,bool isstring=false);
@@ -719,7 +765,6 @@ public:
     void copy(graphe &G) const;
     inline void copy_nodes(const std::vector<vertex> &V);
     inline bool supports_attributes() const { return m_supports_attributes; }
-    void join_edges(const graphe &G);
     void clear();
     void clear_maximal_cliques() { maxcliques.clear(); }
     void find_maximal_cliques();
@@ -743,9 +788,6 @@ public:
     bool is_edge_visited(int i,int j) const;
     inline bool is_edge_visited(const ipair &e) const { return is_edge_visited(e.first,e.second); }
     inline void unvisit_all_edges() { visited_edges.clear(); }
-    void move_neighbor(int v,int w,int ref=-1,bool after=true);
-    template<class Compare>
-    inline void sort_neighbors(int v,Compare comp) { node(v).sort_neighbors(comp); }
     gen to_gen();
     int *to_array(int &sz,bool reduce=false) const;
     bool write_latex(const std::string &filename,const gen &drawing) const;
@@ -753,7 +795,7 @@ public:
     bool read_dot(const std::string &filename);
     inline bool is_null() const { return nodes.empty(); }
     bool is_empty() const;
-    matrice weight_matrix() const;
+    void weight_matrix(matrice &W) const;
     gen weight(int i,int j) const;
     inline gen weight(const ipair &edge) const { return weight(edge.first,edge.second); }
     int edge_count(int sg=-1) const;
@@ -771,7 +813,7 @@ public:
     bool is_triconnected(int sg=-1);
     bool is_cycle(ipairs &E,int sg=-1);
     void adjacent_nodes(int i,ivector &adj,bool include_temp_edges=true) const;
-    void get_edges_as_pairs(ipairs &E, bool include_temp_edges=true,int sg=-1) const;
+    void get_edges_as_pairs(ipairs &E,int sg=-1) const;
     vecteur edges(bool include_weights,int sg=-1) const;
     ivector edge_multiplicities() const;
     int sum_of_edge_multiplicities() const;
@@ -780,10 +822,8 @@ public:
     void add_nodes(const vecteur &v);
     void add_nodes(int n);
     void add_unlabeled_nodes(int n);
-    bool remove_node(int i);
-    bool remove_node(const gen &v);
-    void remove_nodes(const ivector &V);
-    void remove_nodes(const vecteur &V);
+    void remove_isolated_nodes(const iset &I,graphe &G);
+    void isolate_nodes(const iset &V);
     inline const vertex &node(int i) const { return nodes[i]; }
     inline const gen node_label(int i) const { assert(i>=0 && i<node_count()); return nodes[i].label(); }
     vecteur get_node_labels(const ivector &v) const;
@@ -826,6 +866,9 @@ public:
     int in_degree(int index,int sg=-1) const;
     int out_degree(int index,int sg=-1) const;
     int degree(int index,int sg=-1) const;
+    int maximum_degree() const;
+    int minimum_degree() const;
+    int average_degree() const;
     vecteur degree_sequence(int sg=-1) const;
     void sort_by_degrees();
     void adjacency_matrix(matrice &m) const;
@@ -887,20 +930,24 @@ public:
     bool is_tournament() const;
     bool is_planar();
     bool is_clique(int sg=-1) const;
-    int triangle_count() const;
+    gen triangle_count(ivectors *dest=NULL,bool ccoeff=false,bool exact=true);
     int tree_height(int root);
-    void clique_stats(std::map<int,int> &m,bool store_matching=false);
+    void clique_stats(std::map<int,int> &m,int mode=0);
     int maximum_clique(ivector &clique);
     void greedy_neighborhood_clique_cover_numbers(ivector &cover_numbers);
     bool clique_cover(ivectors &cover,int k=0);
     int maximum_independent_set(ivector &v) const;
     int girth(bool odd=false,int sg=-1);
     bool hakimi(const ivector &L);
+    void erdos_renyi(double p);
+    void preferential_attachment(int d,int o);
+    void molloy_reed(const vecteur &p);
     void make_plane_dual(const ivectors &faces);
     void make_lcf_graph(const ivector &jumps,int e);
     void make_lcf_graph(const int *j,int e);
     void make_sierpinski_graph(int n,int k,bool triangle);
     void make_shrikhande_graph();
+    void make_tutte_graph();
     void make_complete_graph();
     void make_complete_multipartite_graph(const ivector &partition_sizes,layout *x=NULL);
     void make_petersen_graph(int n,int k,layout *x=NULL);
@@ -912,14 +959,13 @@ public:
     void make_wheel_graph(int n,layout *x=NULL);
     void make_antiprism_graph(int n,layout *x=NULL);
     void make_complete_kary_tree(int k,int d);
-    void make_random_tree(const vecteur &V,int maxd=0);
-    void make_random_rooted_tree(const vecteur &V);
-    void make_random_free_tree(const vecteur &V);
+    void make_random_tree(int maxd=0);
+    void make_random_rooted_tree();
+    void make_random_free_tree();
     void make_random_planar(double p,int connectivity);
-    void make_random(bool isdir,const vecteur &V,double p);
-    void make_random_sequential(const ivector &d,const vecteur &labels);
-    void make_random_bipartite(const vecteur &V,const vecteur &W,double p);
-    void make_random_regular(const vecteur &V,int d,bool connected);
+    void make_random_sequential(const ivector &d);
+    void make_random_bipartite(int a,int b,double p);
+    void make_random_regular(int d,bool connected);
     static void translate_layout(layout &x,const point &dx);
     void cartesian_product(const graphe &G,graphe &P) const;
     void tensor_product(const graphe &G,graphe &P) const;
@@ -954,7 +1000,8 @@ public:
     void draw_labels(vecteur &drawing,const layout &x) const;
     void distance(int i,const ivector &J,ivector &dist,ivectors *shortest_paths=NULL);
     void allpairs_distance(matrice &m) const;
-    void dijkstra(int src,const ivector &dest,vecteur &path_weights,ivectors *cheapest_paths=NULL);
+    void dijkstra(int src,const ivector &dest,vecteur &path_weights,ivectors *cheapest_paths=NULL,int sg=-1);
+    bool bellman_ford(int src,const ivector &dest,vecteur &path_weights,ivectors *cheapest_paths=NULL);
     bool topologic_sort(ivector &ordering);
     bool is_arborescence() const;
     void reverse(graphe &G) const;
@@ -965,6 +1012,7 @@ public:
     void compute_st_numbering(int s,int t);
     vecteur get_st_numbering() const;
     void assign_edge_directions_from_st();
+    void parametrized_st_orientation(int s,int t,double p);
     void greedy_vertex_coloring_biggs(ivector &ordering);
     int greedy_vertex_coloring(const ivector &p);
     int exact_vertex_coloring(int max_colors=0);
@@ -995,12 +1043,20 @@ public:
     void fundamental_cycles(ivectors &cycles,int sg=-1,bool check=true);
     void mycielskian(graphe &G) const;
     gen local_clustering_coeff(int i) const;
-    gen clustering_coeff() const;
-    gen transitivity() const;
+    gen clustering_coeff(bool approx,bool exact);
+    gen transitivity();
     int edge_connectivity();
     int vertex_connectivity();
+    void truncate(graphe &dest,const ivectors &faces);
+    void condensation(graphe &G);
+    void elementary_cycles(ivectors &cyc,int lo,int hi);
+    void yen_ksp(int K,int src,int dest,ivectors &spaths);
     static gen colon_label(int i,int j);
     static gen colon_label(int i,int j,int k);
+    static size_t intersect_fast(ivector_iter min1,ivector_iter max1,ivector_iter min2,ivector_iter max2);
+    static size_t intersect_linear(ivector_iter min1,ivector_iter max1,ivector_iter min2,ivector_iter max2);
+    static size_t intersect_hybrid(ivector_iter min1,ivector_iter max1,ivector_iter min2,ivector_iter max2);
+    static bool is_graphic_sequence(const ivector &s_orig);
     graphe &operator =(const graphe &other);
 };
 
